#include <Sensor.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>

ESP8266WebServer server(80);

String wifiPoints;
String ssids = "";
String sensors;
String currentSSID;
IPAddress currentIP;
char eepromSSID[32] = "SSID";
char eepromPassword[32] = "password";
char apSSID[20] = "Smart Distiller";
char apPassword[20] = "vp.altukhov.project";
unsigned long millis_loopGetData;
unsigned int disp = 0;
unsigned int sensorCount = 0;
float sensorValue[5] = { 0.0f, 0.0f, 0.0f, 0.0f, 0.0f };

bool scanWiFi(String ssid) {
  WiFi.disconnect(); // обрываем WIFI соединения
  WiFi.softAPdisconnect(); // отключаем отчку доступа(если она была
  WiFi.mode(WIFI_OFF); // отключаем WIFI
  WiFi.mode(WIFI_STA);
  delay(500);

  bool result = false;
  int n = WiFi.scanNetworks();
  ssids = "";
  wifiPoints = "[";
  if (n == 0) {
    wifiPoints = "[]";
  }
  else {
    for (int i = 0; i < n; ++i) {
      if (i > 0) wifiPoints += ",";
      wifiPoints += "{\"Name\":\"" + WiFi.SSID(i) + "\",\"Level\":\"" + WiFi.RSSI(i) + "\"}";
      if (ssid == String(WiFi.SSID(i))) result = true;
      char temp[100];
      snprintf(temp, 100, "<option value=\"%s\">%s</option>", WiFi.SSID(i).c_str(), WiFi.SSID(i).c_str());
      ssids += String(temp);
    }
  }
  wifiPoints = "{\"Points\":" + wifiPoints + "]}";
  return result;
}

void scanSensors() {
  Sensor sensor;
  sensorCount = 0;
  String newSensors = "[";
  while (sensor.searchSensor() && sensorCount < 5) {
    sensorValue[sensorCount] = sensor.getTemperature();
    if (newSensors != "[") newSensors += ",";
    newSensors += sensor.json;
    sensorCount++;
  }
  sensors = "{\"Sensors\":" + newSensors + "]}";
}

void handleRoot() {
  char temp[128];
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;

  snprintf(temp, 128, "{\"Device\":\"%s\",\"Uptime\":\"%02d:%02d:%02d\"}", String(WiFi.macAddress()).c_str(), hr, min % 60, sec % 60);
  server.send(200, "application/json", temp);
}

void handlePing() {
  server.send(200, "application/json", "{\"uid\":\"" + String(WiFi.macAddress()) + "\"}");
}

void handleNetworks() {
  server.send(200, "application/json", wifiPoints);
}

void handleNetwork() {
  char temp[1240];
  
  snprintf ( temp, 1240,
"<html>\
  <head>\
    <title>WiFi setup</title>\
    <style>\
      body { background-color: #cccccc; font-family: Arial, Helvetica, Sans-Serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>WiFi setup</h1>\
    <form action=\"/ssid\">\
      SSID:</br>\
      <select name=\"ssid\" style=\"width:200px;\">\
        %s\
        <option value=\"\">To forget</option>\
      </select>\
      <br><br>\
      Password:</br>\
      <input type=\"password\" name=\"password\"  style=\"width:200px;\" value=\"\"><br><br>\
      <input type=\"submit\" value=\"Submit\">\
    </form>\
  </body>\
</html>",
    ssids.c_str()
  );
  server.send ( 200, "text/html", temp );
}

void handleSensors() {
  server.send(200, "application/json", sensors);
}

void setSSID(String val) {
  for (int i = 0; i < 32; i++) {
    if (i < val.length()) eepromSSID[i] = val[i];
    else eepromSSID[i] = 0;
  }
}

void setPassword(String val) {
  for (int i = 0; i < 32; i++) {
    if (i < val.length()) eepromPassword[i] = val[i];
    else eepromPassword[i] = 0;
  }
}

void handleValue() {
  String json = "{\"Chip\":\"\",\"Code\":\"\",\"Celsius\":\"\"}";
  Sensor sensor;
  for ( uint8_t i = 0; i < server.args(); i++ ) {
    if (server.argName(i) == "uid") {
      while (sensor.searchSensor()) {
        if (sensor.addrStr == server.arg(i)) {
          sensor.getTemperature();
          json = sensor.json;
        }
      }
    }
  }
  server.send ( 200, "application/json", json );
}

void handleSetWiFi() {
  char buf[32] = "";
  EEPROM.begin(64);
  for ( uint8_t i = 0; i < server.args(); i++ ) {
    if (server.argName(i) == "ssid") {
      setSSID(server.arg(i));
      EEPROM.put(0, eepromSSID);
    }
    if (server.argName(i) == "password") {
      setPassword(server.arg(i));
      EEPROM.put(32, eepromPassword);
    }
  }
  EEPROM.end();
  server.send ( 200, "application/json", "\"result\":\"succesful\"" );
  setWiFi();
}

void setWiFi() {
  EEPROM.begin(64);
  //EEPROM.put(0, String("").c_str());
  EEPROM.get(0, eepromSSID);
  EEPROM.get(32, eepromPassword);
  EEPROM.end();
  
  char *ssid = eepromSSID;
  char *password = eepromPassword;
  Serial.println(ssid);
  Serial.println(password);

  bool exist = scanWiFi(String(eepromSSID));
  delay(1000);

  if (exist) {
    WiFi.mode(WIFI_STA);
    delay(500);
    WiFi.begin(ssid, password);
  
    int i = 0;
    while (WiFi.status() != WL_CONNECTED && i < 60) {
      delay(1000);
      i++;
    }
  }

  if (!exist || WiFi.status() != WL_CONNECTED) {
    WiFi.softAP(apSSID, apPassword);
    currentSSID = String(apSSID);
    currentIP = WiFi.softAPIP();
  } else {
    currentSSID = String(ssid);
    currentIP = WiFi.localIP();
  }

  Serial.print("IP address: ");
  Serial.println(currentIP);
  Serial.print("MAC address: ");
  Serial.println(WiFi.macAddress());

  MDNS.begin("esp8266");
  server.onNotFound(handleRoot);
  server.on("/ping", handlePing);
  server.on("/networks", handleNetworks);
  server.on("/sensors", handleSensors);
  server.on("/value", handleValue);
  server.on("/ssid", handleSetWiFi);
  server.on("/network", handleNetwork);
  server.begin();
}

void setup() {
  Serial.begin(115200);
  setWiFi();
}

void loop() {
  if (millis_loopGetData < millis()) {
    millis_loopGetData = millis() + 5000;
    scanSensors();
  }
  server.handleClient();
}
